// Parser.java
//
// Informatics 102 Spring 2012
// Code Example
//
// This is a recursive-descent parser, similar to the one from the previous
// code example.  One key difference is that it makes use of our custom Scanner
// class, which allows it to look at the current token without consuming it.
//
// Note that this parser actually throws exceptions that contain error
// messages, but that as soon as one error is detected, we give up parsing.
// We could certainly do a better job at generating errors and continuing
// to parse than we're doing here, but that's actually a non-trivial problem,
// so we left it out for simplicity.


public class Parser
{
	private Scanner scanner;
	
	
	public Parser(Scanner scanner)
	{
		this.scanner = scanner;
	}
	
	
	public void parse()
	{
		parseProgram();
		
		if (scanner.getCurrentTokenType() != TokenType.END_OF_INPUT)
		{
			throw new ParseException("End of input expected");
		}
	}
	
	
	// Program -> program Identifier '{' DeclarationList '}'
	public void parseProgram()
	{
		expect(TokenType.PROGRAM);
		expect(TokenType.IDENTIFIER);
		expect(TokenType.LEFT_BRACE);
		parseDeclarationList();
		expect(TokenType.RIGHT_BRACE);
	}

	// DeclarationList -> Declaration*
	public void parseDeclarationList()
	{
		// This is an interesting rule, because it's possible for declaration
		// lists to be empty.  How can we possibly detect the case where this
		// DeclarationList is empty?  Since declarations begin with the word
		// "var", we'll check for the presence of the "var" token.  Whenever
		// it's present, we'll read another declaration; when "var" isn't
		// present, we'll return from here, where some other rule can decide
		// whether the declaration list has been followed by the right
		// kind of token (a right brace).

		while (scanner.getCurrentTokenType() == TokenType.VAR)
		{
			parseDeclaration();
		}
	}


	// Declaration -> var Identifier ';'
	public void parseDeclaration()
	{
		expect(TokenType.VAR);
		expect(TokenType.IDENTIFIER);
		expect(TokenType.SEMICOLON);
	}
	
	
	private void expect(TokenType tokenType)
	{
		if (scanner.getCurrentTokenType() == tokenType)
		{
			scanner.advance();
		}
		else
		{
			throw new ParseException(tokenType.toString() + " expected");
		}
	}
}
