// ScanMain.java
//
// Informatics 102 Spring 2012
// Code Example
//
// This program passes all of the input in a file through our Scanner,
// printing a message each time a token is recognized.  This is a handy way
// to test a scanner separately from a parser.
//
// In the event of an exception, the program immediately prints out an error
// and terminates.
//
// This program requires one command-line argument, which specifies the name
// of the input file that should be read.

import java.io.FileReader;
import java.io.IOException;


public class ScanMain
{
	public static void main(String[] args)
	{
		if (args.length != 1)
		{
			printUsageMessage();
			return;
		}

		FileReader fileReader = null;

		try
		{
			fileReader = new FileReader(args[0]);		
			Scanner scanner = new Scanner(fileReader);

			do
			{
				System.out.println(
					"" + scanner.getCurrentTokenType()
					+ " (" + scanner.getCurrentToken() + ")");
				
				scanner.advance();
			}
			while (scanner.getCurrentTokenType() != TokenType.END_OF_INPUT);

			System.out.println(scanner.getCurrentTokenType());
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			if (fileReader != null)
			{
				try
				{
					fileReader.close();
				}
				catch (IOException e)
				{
				}
			}
		}
	}
	
	
	private static void printUsageMessage()
	{
		System.out.println("USAGE: java ScanMain INPUT_FILENAME");
	}
}
